==> nui_core-0.0.255-dev.3.tar.gz: lib/cryptograhy/nui_cryptography.dart <==
import 'dart:convert';
import 'dart:math';
import 'dart:typed_data';

import 'package:cryptography/cryptography.dart';
import 'package:cryptography_flutter/cryptography_flutter.dart';

class NUICryptography {

  final String _key;
  final int _KEY_LENGTH;
  final int _IV_LENGTH;
  final int _TAG_LENGTH;
  final int _SALT_LENGTH;
  final int _KEY_ITERATIONS_COUNT;
  static final Map<String, String> _keys = {};
  static const String defaultKey = "NUIDefaultKey";

  static NUICryptography _instance;

  NUICryptography._builder(NUICryptographyBuilder builder) :
      _key = builder._defaultKey,
      _KEY_LENGTH = builder._KEY_LENGTH ?? 32,
      _IV_LENGTH = builder._IV_LENGTH ?? 12,
      _TAG_LENGTH = builder._TAG_LENGTH ?? 16,
      _SALT_LENGTH = builder._SALT_LENGTH ?? 16,
      _KEY_ITERATIONS_COUNT = builder._KEY_ITERATIONS_COUNT ?? 10000{
    if(utf8.encode(_key).length != _KEY_LENGTH) throw Exception("NUICryptography Build Failed => Key/Password  <$_key> Length is not $_KEY_LENGTH");
      _keys[defaultKey] = _key;
  }

  static NUICryptography get(){
    if(_instance == null) throw Exception("NUICryptography -> NUICryptography instance has not been initialized with builder");
    return _instance;
  }

  Uint8List randomBytes(int length) {
    Uint8List buffer = Uint8List(length);
    Random range = Random.secure();

    for (int i = 0; i < length; i++) {
      buffer[i] = range.nextInt(256);
    }

    return buffer;
  }

  void setNewKey(String keyName, String newKey){
    if(utf8.encode(newKey).length != _KEY_LENGTH) throw Exception("NUICryptography Build Failed => Key/Password  <$newKey> Length is not $_KEY_LENGTH");
    if(_keys.containsKey(keyName)) throw Exception("NUICryptography => $keyName already existed");
    _keys[keyName] = newKey;
  }

  Future<SecretKey> deriveKey(String password, List<int> salt) async {
    final pbkdf2 = Pbkdf2(
      macAlgorithm: Hmac.sha256(),
      iterations: _KEY_ITERATIONS_COUNT,
      bits: _KEY_LENGTH * 8,
    );

    SecretKey secret = await pbkdf2.deriveKey(
      secretKey: SecretKey(utf8.encode(password)),
      nonce: salt,
    );

    return secret;
  }

  Future<String> encryptWithAesGcm256(String cleartext, {String keyName = defaultKey}) async {
    final salt = randomBytes(_SALT_LENGTH);
    final iv = randomBytes(_IV_LENGTH);
    final key = await deriveKey(_keys[keyName], salt);
    final algorithm = FlutterAesGcm(AesGcm.with256bits());

    final secretBox = await algorithm.encrypt(
      utf8.encode(cleartext),
      secretKey: key,
      nonce: iv,
    );

    final List<int> result = salt + secretBox.nonce + secretBox.cipherText + secretBox.mac.bytes;
    return base64.encode(result);
  }

  Future<String> decryptWithAesGcm256(String cipherText, {String keyName = defaultKey}) async {
    final cText = base64.decode(cipherText);
    final salt = cText.sublist(0, _SALT_LENGTH);
    final iv = cText.sublist(_SALT_LENGTH, _SALT_LENGTH + _IV_LENGTH);
    final mac = cText.sublist(cText.length - _TAG_LENGTH);
    final text = cText.sublist(_SALT_LENGTH + _IV_LENGTH, cText.length - _TAG_LENGTH);

    final algorithm = FlutterAesGcm(AesGcm.with256bits());
    final key = await deriveKey(_keys[keyName], salt);

    final secretBox = SecretBox(text, nonce: iv, mac: Mac(mac));

    final cleartext = await algorithm.decrypt(
      secretBox,
      secretKey: key,
    );

    return utf8.decode(cleartext);
  }


}

class NUICryptographyBuilder{

  int _KEY_LENGTH;
  int _IV_LENGTH;
  int _TAG_LENGTH;
  int _SALT_LENGTH;
  int _KEY_ITERATIONS_COUNT;
  String _defaultKey;

  NUICryptographyBuilder keyLength(int length){
    _KEY_LENGTH = length;
    return this;
  }

  NUICryptographyBuilder ivLength(int length){
    _IV_LENGTH = length;
    return this;
  }

  NUICryptographyBuilder tagLength(int length){
    _TAG_LENGTH = length;
    return this;
  }

  NUICryptographyBuilder saltLength(int length){
    _SALT_LENGTH = length;
    return this;
  }

  NUICryptographyBuilder iterations(int count){
    _KEY_ITERATIONS_COUNT = count;
    return this;
  }

  NUICryptographyBuilder setDefaultKey(String key){
    _defaultKey = key;
    return this;
  }

  Future<NUICryptography> build() async{
    final crypto = NUICryptography._builder(this);
    // logNUI("NUICryptography", "Initialization successful");
    NUICryptography._instance = crypto;
    return crypto;
  }

}

==> nui_core-1.0.1.tar.gz: lib/cryptograhy/nui_cryptography.dart <==
import 'dart:convert';
import 'dart:math';
import 'dart:typed_data';
import 'package:cryptography/cryptography.dart';
import 'package:cryptography_flutter/cryptography_flutter.dart';

class NUICryptography {

  final String _key;
  final int _KEY_LENGTH;
  final int _IV_LENGTH;
  final int _TAG_LENGTH;
  final int _SALT_LENGTH;
  final int _KEY_ITERATIONS_COUNT;
  static final Map<String, String> _keys = {};
  static const String defaultKey = "NUIDefaultKey";

  static NUICryptography? _instance;

  NUICryptography._builder(NUICryptographyBuilder builder) :
      _key = builder._defaultKey,
      _KEY_LENGTH = builder._KEY_LENGTH ?? 32,
      _IV_LENGTH = builder._IV_LENGTH ?? 12,
      _TAG_LENGTH = builder._TAG_LENGTH ?? 16,
      _SALT_LENGTH = builder._SALT_LENGTH ?? 16,
      _KEY_ITERATIONS_COUNT = builder._KEY_ITERATIONS_COUNT ?? 10000{
    if(utf8.encode(_key).length != _KEY_LENGTH) throw Exception("NUICryptography Build Failed => Key/Password  <$_key> Length is not $_KEY_LENGTH");
      _keys[defaultKey] = _key;
  }

  static NUICryptography? get(){
    if(_instance == null) throw Exception("NUICryptography -> NUICryptography instance has not been initialized with builder");
    return _instance;
  }

  Uint8List randomBytes(int length) {
    Uint8List buffer = Uint8List(length);
    Random range = Random.secure();

    for (int i = 0; i < length; i++) {
      buffer[i] = range.nextInt(256);
    }

    return buffer;
  }

  void setNewKey(String keyName, String newKey){
    if(utf8.encode(newKey).length != _KEY_LENGTH) throw Exception("NUICryptography Build Failed => Key/Password  <$newKey> Length is not $_KEY_LENGTH");
    if(_keys.containsKey(keyName)) throw Exception("NUICryptography => $keyName already existed");
    _keys[keyName] = newKey;
  }

  Future<SecretKey> deriveKey(String password, List<int> salt) async {
    final pbkdf2 = Pbkdf2(
      macAlgorithm: Hmac.sha256(),
      iterations: _KEY_ITERATIONS_COUNT,
      bits: _KEY_LENGTH * 8,
    );

    SecretKey secret = await pbkdf2.deriveKey(
      secretKey: SecretKey(utf8.encode(password)),
      nonce: salt,
    );

    return secret;
  }

  Future<String> encryptWithAesGcm256(String cleartext, {String keyName = defaultKey}) async {
    final salt = randomBytes(_SALT_LENGTH);
    final iv = randomBytes(_IV_LENGTH);
    final key = await deriveKey(_keys[keyName]!, salt);
    final algorithm = FlutterAesGcm(AesGcm.with256bits());

    final secretBox = await algorithm.encrypt(
      utf8.encode(cleartext),
      secretKey: key,
      nonce: iv,
    );

    final List<int> result = salt + secretBox.nonce + secretBox.cipherText + secretBox.mac.bytes;
    return base64.encode(result);
  }

  Future<String> decryptWithAesGcm256(String cipherText, {String keyName = defaultKey}) async {
    final cText = base64.decode(cipherText);
    final salt = cText.sublist(0, _SALT_LENGTH);
    final iv = cText.sublist(_SALT_LENGTH, _SALT_LENGTH + _IV_LENGTH);
    final mac = cText.sublist(cText.length - _TAG_LENGTH);
    final text = cText.sublist(_SALT_LENGTH + _IV_LENGTH, cText.length - _TAG_LENGTH);

    final algorithm = FlutterAesGcm(AesGcm.with256bits());
    final key = await deriveKey(_keys[keyName]!, salt);

    final secretBox = SecretBox(text, nonce: iv, mac: Mac(mac));

    final cleartext = await algorithm.decrypt(
      secretBox,
      secretKey: key,
    );

    return utf8.decode(cleartext);
  }


}

class NUICryptographyBuilder{
  late int _KEY_LENGTH;
  late int _IV_LENGTH;
  late int _TAG_LENGTH;
  late int _SALT_LENGTH;
  late int _KEY_ITERATIONS_COUNT;
  late String _defaultKey;

  NUICryptographyBuilder keyLength(int length){
    _KEY_LENGTH = length;
    return this;
  }

  NUICryptographyBuilder ivLength(int length){
    _IV_LENGTH = length;
    return this;
  }

  NUICryptographyBuilder tagLength(int length){
    _TAG_LENGTH = length;
    return this;
  }

  NUICryptographyBuilder saltLength(int length){
    _SALT_LENGTH = length;
    return this;
  }

  NUICryptographyBuilder iterations(int count){
    _KEY_ITERATIONS_COUNT = count;
    return this;
  }

  NUICryptographyBuilder setDefaultKey(String key){
    _defaultKey = key;
    return this;
  }

  Future<NUICryptography> build() async{
    final crypto = NUICryptography._builder(this);
    // logNUI("NUICryptography", "Initialization successful");
    NUICryptography._instance = crypto;
    return crypto;
  }

}
